==> lodestone-1.21.1-1.7.0.317-sources.jar: team/lodestar/lodestone/systems/network/particle/NetworkedParticleEffectColorData.java <==
package team.lodestar.lodestone.systems.network.particle;

import com.mojang.serialization.Codec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import io.netty.buffer.ByteBuf;
import net.minecraft.network.codec.ByteBufCodecs;
import net.minecraft.network.codec.StreamCodec;
import net.minecraft.resources.ResourceLocation;
import team.lodestar.lodestone.systems.particle.data.color.ColorParticleData;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;

public class NetworkedParticleEffectColorData {

    public static final Codec<NetworkedParticleEffectColorData> CODEC = RecordCodecBuilder.create(instance -> instance.group(
            ColorParticleData.CODEC.listOf().fieldOf("colors").forGetter(data -> data.colors)
    ).apply(instance, NetworkedParticleEffectColorData::new));

    public static final StreamCodec<ByteBuf, NetworkedParticleEffectColorData> STREAM_CODEC = ByteBufCodecs.fromCodec(CODEC);

    protected final List<ColorParticleData> colors;
    protected int colorCycleCounter;

    public static NetworkedParticleEffectColorData fromColors(List<ColorParticleData> colors) {
        return new NetworkedParticleEffectColorData(colors);
    }

    public static NetworkedParticleEffectColorData fromColor(ColorParticleData color) {
        return fromColors(List.of(color));
    }

    public NetworkedParticleEffectColorData(List<ColorParticleData> colors) {
        this.colors = colors.isEmpty() ? Collections.emptyList() : colors;
    }

    public NetworkedParticleEffectColorData(ColorParticleData... colors) {
        this(List.of(colors));
    }

    public ColorParticleData getColor() {
        if (colors.size() == 1) {
            return colors.getFirst();
        }
        return colors.get(colorCycleCounter++ % colors.size());
    }
}

==> lodestone-1.21.1-1.7.2.393-sources.jar: team/lodestar/lodestone/systems/network/particle/NetworkedParticleEffectColorData.java <==
package team.lodestar.lodestone.systems.network.particle;

import com.mojang.serialization.Codec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import io.netty.buffer.ByteBuf;
import net.minecraft.network.codec.ByteBufCodecs;
import net.minecraft.network.codec.StreamCodec;
import net.minecraft.resources.ResourceLocation;
import team.lodestar.lodestone.systems.particle.data.color.*;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;

public class NetworkedParticleEffectColorData {

    public static final Codec<NetworkedParticleEffectColorData> CODEC = RecordCodecBuilder.create(instance -> instance.group(
            ColorParticleData.CODEC.listOf().fieldOf("colors").forGetter(data -> data.colors)
    ).apply(instance, NetworkedParticleEffectColorData::new));

    public static final StreamCodec<ByteBuf, NetworkedParticleEffectColorData> STREAM_CODEC = ByteBufCodecs.fromCodec(CODEC);

    protected final List<ColorParticleData> colors;
    protected int colorCycleCounter;

    public static NetworkedParticleEffectColorData fromColors(List<? extends ColorParticleDataWrapper> colors) {
        return new NetworkedParticleEffectColorData(colors);
    }

    public static NetworkedParticleEffectColorData fromColor(ColorParticleDataWrapper color) {
        return fromColors(List.of(color));
    }

    public NetworkedParticleEffectColorData(List<? extends ColorParticleDataWrapper> colors) {
        this.colors = colors.isEmpty() ? Collections.emptyList() : colors.stream().map(ColorParticleDataWrapper::unwrap).toList();
    }

    public NetworkedParticleEffectColorData(ColorParticleDataWrapper... colors) {
        this(List.of(colors));
    }

    public ColorParticleData getColor() {
        if (colors.size() == 1) {
            return colors.getFirst();
        }
        return colors.get(colorCycleCounter++ % colors.size());
    }

    public List<ColorParticleData> getColors() {
        return colors;
    }
}
